from ajenti.api import *
from ajenti.licensing import Licensing
from ajenti.plugins import manager
from ajenti.ui import UIElement, on


@plugin
class LicensingUI (UIElement):
    typeid = 'configurator:licensingui'

    def init(self):
        self.append(self.ui.inflate('configurator:licensing'))
        self.mgr = Licensing.get()

    def on_page_load(self):
        self.refresh()

    def on_tab_shown(self):
        self.refresh()

    def refresh(self):
        license_status = self.mgr.get_license_status()
        active = bool(self.mgr.get_license_status())
        self.find('license-active').visible = active
        self.find('license-inactive').visible = not active

        self.find('license-current-status').text = {
            'ok': 'OK',
            'invalid-key': _('Invalid key'),
            'invalid-ip': _('Invalid IP'),
            'expired': _('Expired'),
        }.get(license_status.get('status', None), _('Unknown'))

        license = license_status.get('license', {})
        self.find('license-current-expires').text = license.get('expires', '')
        self.find('license-current-type').text = license.get('type', '')
        self.find('license-current-company').text = license.get('company', '')

    @on('license-install', 'click')
    def on_install(self):
        try:
            self.mgr.write_license(self.find('license-key').value)
            self.mgr.activate()
        except Exception as e:
            self.context.notify('error', _('Error: "%s"') % str(e))
        self.refresh()

    @on('license-remove', 'click')
    def on_remove(self):
        try:
            self.mgr.deactivate()
            self.mgr.remove_license()
        except Exception as e:
            self.context.notify('error', _('Error: "%s"') % str(e))
        self.refresh()


import os
import gzip
import cgi
import math
import gevent
from StringIO import StringIO
from datetime import datetime

from socketio.handler import SocketIOHandler


def _validate_origin(env):
    valid_origin = '%s://%s' % (env['wsgi.url_scheme'], env['HTTP_HOST'])
    request_origin = env.get('HTTP_ORIGIN', '').strip('/')
    if request_origin:
        if request_origin != valid_origin:
            return False
    return True


class RootHttpHandler (SocketIOHandler):
    def handle_one_response(self):
        if not _validate_origin(self.environ):
            return super(SocketIOHandler, self).handle_one_response()
        return SocketIOHandler.handle_one_response(self)


class HttpRoot (object):
    """
    A root middleware object that creates the :class:`HttpContext` and dispatches it to other registered middleware
    """

    def __init__(self, stack=[]):
        self.stack = stack

    def add(self, middleware):
        """
        Pushes the middleware onto the stack
        """
        self.stack.append(middleware)

    def dispatch(self, env, start_response):
        """
        Dispatches the WSGI request
        """
        if not _validate_origin(env):
            start_response('403 Invalid Origin', [])
            return ''

        context = HttpContext(env, start_response)
        for middleware in self.stack:
            output = middleware.handle(context)
            if output is not None:
                return output


class HttpContext (object):
    """
    Instance of :class:`HttpContext` is passed to all HTTP handler methods

    .. attribute:: env

        WSGI environment dict

    .. attribute:: path

        Path segment of the URL

    .. attribute:: headers

        List of HTTP response headers

    .. attribute:: response_ready

        Indicates whether a HTTP response has already been submitted in this context

    .. attribute:: query

        HTTP query parameters
    """

    def __init__(self, env, start_response):
        self.start_response = start_response
        self.env = env
        self.path = env['PATH_INFO']
        self.headers = []
        self.response_ready = False

        self.env.setdefault('QUERY_STRING', '')
        if self.env['REQUEST_METHOD'].upper() == 'POST':
            ctype = self.env.get('CONTENT_TYPE', 'application/x-www-form-urlencoded')
            if ctype.startswith('application/x-www-form-urlencoded') \
               or ctype.startswith('multipart/form-data'):
                sio = StringIO(self.env['wsgi.input'].read())
                self.query = cgi.FieldStorage(fp=sio, environ=self.env, keep_blank_values=1)
            else:
                self.body = self.env['wsgi.input'].read()
        else:
            self.query = cgi.FieldStorage(environ=self.env, keep_blank_values=1)

    def add_header(self, key, value):
        """
        Adds a given HTTP header to the response

        :type key: str
        :type value: str
        """
        self.headers += [(key, value)]

    def remove_header(self, key):
        """
        Removed a given HTTP header from the response

        :type key: str
        """
        self.headers = filter(lambda h: h[0] != key, self.headers)

    def fallthrough(self, handler):
        """
        Executes a ``handler`` in this context

        :returns: handler-supplied output
        """
        return handler.handle(self)

    def respond(self, status):
        """
        Creates a response with given HTTP status line
        :type status: str
        """
        self.start_response(status, self.headers)
        self.response_ready = True

    def respond_ok(self):
        """
        Creates a ``HTTP 200 OK`` response
        """
        self.respond('200 OK')

    def respond_server_error(self):
        """
        Returns a HTTP ``500 Server Error`` response
        """
        self.respond('500 Server Error')
        return 'Server Error'

    def respond_forbidden(self):
        """
        Returns a HTTP ``403 Forbidden`` response
        """
        self.respond('403 Forbidden')
        return 'Forbidden'

    def respond_not_found(self):
        """
        Returns a ``HTTP 404 Not Found`` response
        """
        self.respond('404 Not Found')
        return 'Not Found'

    def redirect(self, url):
        """
        Returns a ``HTTP 302 Found`` redirect response with given ``url``
        :type url: str
        """
        self.add_header('Location', url)
        self.respond('302 Found')
        return ''

    def gzip(self, content, compression=9):
        """
        Returns a GZip compressed response with given ``content`` and correct headers
        :type content: str
        :type compression: int
        :rtype: str
        """
        io = StringIO()
        gz = gzip.GzipFile('', 'wb', compression, io)
        gz.write(content)
        gz.close()
        compressed = io.getvalue()

        self.add_header('Content-Length', str(len(compressed)))
        self.add_header('Content-Encoding', 'gzip')
        self.respond_ok()

        return compressed

    def file(self, path, stream=False):
        """
        Returns a GZip compressed response with content of file located in ``path`` and correct headers
        :type path: str
        :type stream: bool
        """

        # Block path traversal
        if '..' in path:
            self.respond_forbidden()
            return

        if not os.path.isfile(path):
            self.respond_not_found()
            return

        content_types = {
            '.css': 'text/css',
            '.js': 'application/javascript',
            '.png': 'image/png',
            '.jpg': 'image/jpeg',
            '.woff': 'application/x-font-woff',
        }

        ext = os.path.splitext(path)[1]
        if ext in content_types:
            self.add_header('Content-Type', content_types[ext])
        else:
            self.add_header('Content-Type', 'application/octet-stream')

        mtime = datetime.utcfromtimestamp(math.trunc(os.path.getmtime(path)))

        rtime = self.env.get('HTTP_IF_MODIFIED_SINCE', None)
        if rtime:
            try:
                rtime = datetime.strptime(rtime, '%a, %b %d %Y %H:%M:%S GMT')
                if mtime <= rtime:
                    self.respond('304 Not Modified')
                    return
            except:
                pass

        range = self.env.get('HTTP_RANGE', None)
        rfrom = rto = None
        if range and range.startswith('bytes'):
            rsize = os.stat(path).st_size
            rfrom, rto = range.split('=')[1].split('-')
            rfrom = int(rfrom) if rfrom else 0
            rto = int(rto) if rto else (rsize - 1)
        else:
            rfrom = 0
            rto = 999999999

        self.add_header('Last-Modified', mtime.strftime('%a, %b %d %Y %H:%M:%S GMT'))
        self.add_header('Accept-Ranges', 'bytes')

        if stream:
            if rfrom:
                self.add_header('Content-Length', str(rto - rfrom + 1))
                self.add_header('Content-Range', 'bytes %i-%i/%i' % (rfrom, rto, rsize))
                self.respond('206 Partial Content')
            else:
                self.respond_ok()
            fd = os.open(path, os.O_RDONLY)# | os.O_NONBLOCK)
            os.lseek(fd, rfrom or 0, os.SEEK_SET)
            bufsize = 100 * 1024
            read = rfrom
            buf = 1
            while buf:
                buf = os.read(fd, bufsize)
                gevent.sleep(0)
                if read + len(buf) > rto:
                    buf = buf[:rto + 1 - read]
                yield buf
                read += len(buf)
                if read >= rto:
                    break
            os.close(fd)
        else:
            yield self.gzip(open(path).read())


class HttpHandler (object):
    """
    Base class for everything that can process HTTP requests
    """

    def handle(self, context):
        """
        Should create a HTTP response in the given ``context`` and return the plain output

        :param context: HTTP context
        :type  context: :class:`ajenti.http.HttpContext`
        """



import subprocess

from ajenti.api import plugin, interface
from ajenti.util import cache_value


@interface
class PowerController (object):
    def shutdown(self):
        pass

    def suspend(self):
        pass

    def hibernate(self):
        pass

    def reboot(self):
        pass

    @classmethod
    def capabilities(cls):
        pass


@plugin
class SystemdPowerController (PowerController):
    def shutdown(self):
        subprocess.call(['systemctl', 'poweroff'])

    def reboot(self):
        subprocess.call(['systemctl', 'reboot'])

    def suspend(self):
        subprocess.call(['systemctl', 'suspend'])

    def hibernate(self):
        subprocess.call(['systemctl', 'hibernate'])

    @classmethod
    @cache_value()
    def capabilities(cls):
        if subprocess.call(['which', 'systemctl']) == 0:
            return ['reboot', 'suspend', 'hibernate', 'shutdown']
        else:
            return []

    @classmethod
    def verify(cls):
        return subprocess.call(['which', 'systemctl']) == 0


@plugin
class PMUtilsPowerController (PowerController):
    def shutdown(self):
        subprocess.call(['poweroff'])

    def suspend(self):
        subprocess.call(['pm-suspend'])

    def hibernate(self):
        subprocess.call(['pm-hibernate'])

    def reboot(self):
        subprocess.call(['reboot'])

    @classmethod
    @cache_value()
    def capabilities(cls):
        return ['shutdown', 'reboot', 'suspend', 'hibernate']

    @classmethod
    def verify(cls):
        return subprocess.call(['which', 'pm-hibernate']) == 0


@plugin
class BasicLinuxPowerController (PowerController):
    def shutdown(self):
        subprocess.call(['poweroff'])

    def reboot(self):
        subprocess.call(['reboot'])

    @classmethod
    def capabilities(cls):
        return ['shutdown', 'reboot']



import os
import re
import psutil
import shutil
import statvfs

from ajenti.api import *
from ajenti.api.sensors import Sensor


def list_devices(by_name=True, by_uuid=False, by_id=False, by_label=False):
    result = []

    def add_dir(path, namefx=lambda s, r: s, valuefx=lambda s, r: r):
        if os.path.exists(path):
            for s in os.listdir(path):
                rp = os.path.realpath(os.path.join(path, s))
                result.append((namefx(s, rp), valuefx(s, rp)))

    for s in os.listdir('/dev'):
        if re.match('sd.$|hd.$|scd.$|fd.$|ad.+$', s):
            result.append((s, '/dev/' + s))

    if by_uuid:
        add_dir('/dev/disk/by-uuid', lambda s, r: '%s: UUID %s' % (r, s), lambda s, r: 'UUID=%s' % s)

    if by_label:
        add_dir('/dev/disk/by-label', lambda s, r: 'Label "%s"' % s, lambda s, r: 'LABEL=%s' % s)

    if by_id:
        add_dir('/dev/disk/by-id', lambda s, r: '%s: %s' % (r, s))

    return sorted(result, key=lambda x: x[0])


@plugin
class PSUtilDiskUsageSensor (Sensor):
    id = 'disk-usage'
    timeout = 5

    @classmethod
    def __list_partitions(cls):
        return filter(lambda p: p.device, psutil.disk_partitions(True))
    
    @classmethod
    def verify(cls):
        try:
            return len(cls.__list_partitions()) > 0
        except:
            return False

    def get_variants(self):
        return sorted([x.mountpoint for x in self.__list_partitions()])

    def measure(self, path):
        try:
            if path:
                v = psutil.disk_usage(path)
            else:
                return (0, 1)
        except OSError:
            return (0, 1)
        return (v.used, v.free, v.total)


@plugin
class MTabDiskUsageSensor (Sensor):
    """
    Useful for procfs-less virtualization containers
    """
    id = 'disk-usage'
    timeout = 5

    @classmethod
    def verify(cls):
        return os.path.exists('/etc/mtab')

    def get_variants(self):
        return filter(lambda x: x != 'none', sorted(l.split()[1] for l in open('/etc/mtab')))

    def measure(self, path):
        s = os.statvfs(path)
        total = s[statvfs.F_FRSIZE] * s[statvfs.F_BLOCKS]
        free = s[statvfs.F_BFREE] * s[statvfs.F_BSIZE]
        return (total - free, free, total)
        


from ajenti.api import *
from ajenti.ui import *


@p('title')
@p('icon', default=None)
@p('order', default=99, doc='Sorting weight, light plugins end up higher')
@p('category', default='Other', doc='Section category name')
@p('active', default=False)
@p('clsname', default='')
@p('hidden', default=False)
@track
@interface
class SectionPlugin (BasePlugin, UIElement):
    """
    A base class for section plugins visible on the left in the UI. Inherits :class:`ajenti.ui.UIElement`
    """

    typeid = 'main:section'
    hidden = False

    def init(self):
        self._first_page_load = True
        self._intents = {}
        for k, v in self.__class__.__dict__.iteritems():
            if hasattr(v, '_intent'):
                self._intents[v._intent] = getattr(self, k)

    def activate(self):
        """
        Activate this section
        """
        self.context.endpoint.switch_tab(self)

    def on_first_page_load(self):
        """
        Called before first ``on_page_load``
        """

    def on_page_load(self):
        """
        Called when this section becomes active, or the page is reloaded
        """


def intent(id):
    """
    Registers this method as an intent with given ID
    """
    def decorator(fx):
        fx._intent = id
        return fx
    return decorator



import dbus
import logging
import subprocess

from ajenti.api import *
from ajenti.util import cache_value

from api import Service, ServiceManager


@plugin
class UpstartServiceManager (ServiceManager):
    def init(self):
        self.bus = dbus.SystemBus()
        self.upstart = self.bus.get_object("com.ubuntu.Upstart", "/com/ubuntu/Upstart")

    @classmethod
    def verify(cls):
        try:
            c = cls()
            c.init()
            return True
        except Exception, e:
            logging.info('Disabling Upstart service manager: %s' % str(e))
            return False

    @cache_value(1)
    def get_all(self):
        jobs = self.upstart.GetAllJobs(dbus_interface="com.ubuntu.Upstart0_6")
        r = []
        for job in jobs:
            obj = self.bus.get_object("com.ubuntu.Upstart", job)
            jprops = obj.GetAll("com.ubuntu.Upstart0_6.Job", dbus_interface=dbus.PROPERTIES_IFACE)

            s = UpstartService(str(jprops['name']))

            paths = obj.GetAllInstances(dbus_interface="com.ubuntu.Upstart0_6.Job")
            if len(paths) > 0:
                instance = self.bus.get_object("com.ubuntu.Upstart", paths[0])
                iprops = instance.GetAll("com.ubuntu.Upstart0_6.Instance", dbus_interface=dbus.PROPERTIES_IFACE)
                s.running = str(iprops['state']) == 'running'
            else:
                s.running = False

            r.append(s)
        return r


class UpstartService (Service):
    source = 'upstart'

    def __init__(self, name):
        self.name = name

    def refresh(self):
        self.running = 'running' in subprocess.check_output(['status', self.name])

    def start(self):
        self.command('start')

    def stop(self):
        self.command('stop')

    def restart(self):
        self.command('restart')

    def command(self, cmd):
        subprocess.Popen(['initctl', cmd, self.name], close_fds=True).wait()



import gevent
from gevent.pywsgi import WSGIServer, WSGIHandler
import logging
import os
import socket
import threading
import traceback

from ajenti.api import *
from ajenti.plugins import manager
from ajenti.util import public


@public
@rootcontext
@interface
class IPCHandler (object):
    """
    Interface for custom IPC endpoints
    """

    def get_name(self):
        """
        Should return short identifier of IPC endpoint:

        $ ajenti-ipc <endpoint-name> <args>

        :rtype str:
        """

    def handle(self, args):
        """
        Override to handle IPC requests

        :param args: list of `str` parameters
        :type  args: list
        """


class IPCWSGIHandler (WSGIHandler):
    def __init__(self, *args, **kwargs):
        WSGIHandler.__init__(self, *args, **kwargs)
        self.client_address = ('ipc', 0)

    def log_request(self):
        pass


class IPCSocketServer (WSGIServer):
    pass


def ipc_application(environment, start_response):
    name, args = environment['PATH_INFO'].split('/')
    args = args.decode('base64').splitlines()
    logging.info('IPC: %s %s' % (name, args))

    for h in IPCHandler.get_all(manager.context):
        if h.get_name() == name:
            try:
                result = h.handle(args)
                if result is None:
                    start_response('404 Not found', [])
                    return ''
                else:
                    start_response('200 OK', [])
                    return result
            except Exception as e:
                traceback.print_exc()
                start_response('500 Error', [])
                return str(e)
            break
    else:
        start_response('404 Handler not found', [])


@public
@plugin
@persistent
@rootcontext
class IPCServer (BasePlugin):
    def start(self):
        gevent.spawn(self.run)

    def run(self):
        socket_path = '/var/run/ajenti-ipc.sock'
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        if os.path.exists(socket_path):
            os.unlink(socket_path)
        sock.bind(socket_path)
        sock.listen(5)
        os.chmod(socket_path, 0700)

        server = IPCSocketServer(sock, application=ipc_application, handler_class=IPCWSGIHandler)
        server.serve_forever()



try:
    import ldap
except ImportError:
    ldap = None

import ajenti
from ajenti.api import *
from ajenti.plugins.configurator.api import ClassConfigEditor

from reconfigure.items.ajenti import UserData

from base import UserSyncProvider



@plugin
class ADSyncClassConfigEditor (ClassConfigEditor):
    title = _('Active Directory Syncronization')
    icon = 'refresh'

    def init(self):
        self.append(self.ui.inflate('configurator:ad-sync-config'))


@plugin
class ActiveDirectorySyncProvider (UserSyncProvider, BasePlugin):
    id = 'ad'
    title = 'Active Directory'
    default_classconfig = {
        'address': 'localhost',
        'domain': 'DOMAIN',
        'user': 'Administrator',
        'password': '',
        'base': 'cn=Users,dc=DOMAIN',
    }
    classconfig_root = True
    classconfig_editor = ADSyncClassConfigEditor

    @classmethod
    def verify(cls):
        return ldap is not None

    def __get_ldap(self):
        if not ldap:
            return None

        c = ldap.initialize('ldap://' + self.classconfig['address'])
        c.bind_s('%s\\%s' % (self.classconfig['domain'], self.classconfig.get('user', 'Administrator')), self.classconfig['password'])
        c.set_option(ldap.OPT_REFERRALS, 0)
        return c

    def test(self):
        self.__search()

    def check_password(self, username, password):
        l = self.__get_ldap()
        try:
            return bool(l.bind_s('%s\\%s' % (self.classconfig['domain'], username), password))
        except Exception as e:
            print(e)
            return False

    def __search(self):
        l = self.__get_ldap()
        return l.search_s(
            self.classconfig['base'],
            ldap.SCOPE_SUBTREE,
            '(|(objectClass=user)(objectClass=simpleSecurityObject))',
            ['sAMAccountName']
        )

    def sync(self):
        found_names = []
        users = self.__search()
        for u in users:
            username = u[1]['sAMAccountName'][0]
            found_names.append(username)
            if not username in ajenti.config.tree.users:
                u = UserData()
                u.name = username
                ajenti.config.tree.users[username] = u

        for user in list(ajenti.config.tree.users.values()):
            if not user.name in found_names and user.name != 'root':
                ajenti.config.tree.users.pop(user.name)

        ajenti.config.save()



import gevent

from ajenti.api import *
from ajenti.plugins.packages.api import PackageManager, PackageInfo


@plugin
class AjentiUpdater (BasePlugin):
    AJENTI_PACKAGE_NAME = 'ajenti'

    def run_update(self, packages):
        packages = packages or [self.AJENTI_PACKAGE_NAME]
        actions = []
        for name in packages:
            mgr = PackageManager.get()
            p = PackageInfo()
            p.name, p.action = name, 'i'
            actions.append(p)
        mgr.do(actions)

    def check_for_updates(self, callback):
        try:
            mgr = PackageManager.get()
        except NoImplementationsError:
            return

        def worker():
            mgr.refresh()
            r = []
            for p in mgr.upgradeable:
                if p.name.startswith(self.AJENTI_PACKAGE_NAME):
                    r.append(p.name)
            callback(r)

        gevent.spawn(worker)



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='LM-Sensors',
    description='lm-sensors temperature sensor widgets',
    icon='leaf',
    dependencies=[
        PluginDependency('main'),
        PluginDependency('dashboard'),
        BinaryDependency('sensors'),
    ],
)


def init():
    import main



import grp
import logging
import os
import pwd
import re
import subprocess
import stat
import shutil
from datetime import datetime

from ajenti.api import *
from ajenti.util import str_fsize
from ajenti.plugins import manager
from ajenti.plugins.tasks.manager import TaskManager
from ajenti.plugins.tasks.tasks import CopyFilesTask, MoveFilesTask, DeleteFilesTask


class Item (object):
    stat_bits = [
        stat.S_IRUSR,
        stat.S_IWUSR,
        stat.S_IXUSR,
        stat.S_IRGRP,
        stat.S_IWGRP,
        stat.S_IXGRP,
        stat.S_IROTH,
        stat.S_IWOTH,
        stat.S_IXOTH,
    ]

    def __init__(self, path):
        self.checked = False
        self.path, self.name = os.path.split(path)
        self.fullpath = path
        self.isdir = os.path.isdir(path)
        self.icon = 'folder-close' if self.isdir else 'file'
        try:
            self.size = 0 if self.isdir else os.path.getsize(path)
        except OSError:
            self.size = 0
        self.sizestr = '' if self.isdir else str_fsize(self.size)
        self.mtime = datetime.utcfromtimestamp(os.stat(path).st_mtime)
        self.atime = datetime.utcfromtimestamp(os.stat(path).st_atime)
        self.ctime = datetime.utcfromtimestamp(os.stat(path).st_ctime)

    def read(self):
        stat = os.stat(self.fullpath)

        try:
            self.owner = pwd.getpwuid(stat.st_uid)[0]
        except KeyError:
            self.owner = str(stat.st_uid)

        try:
            self.group = grp.getgrgid(stat.st_gid)[0]
        except KeyError:
            self.group = str(stat.st_gid)

        self.mod_ur, self.mod_uw, self.mod_ux, \
            self.mod_gr, self.mod_gw, self.mod_gx, \
            self.mod_ar, self.mod_aw, self.mod_ax = [
                (stat.st_mode & Item.stat_bits[i] != 0)
                for i in range(0, 9)
            ]

    @property
    def mode(self):
        mods = [
            self.mod_ur, self.mod_uw, self.mod_ux,
            self.mod_gr, self.mod_gw, self.mod_gx,
            self.mod_ar, self.mod_aw, self.mod_ax
        ]
        return sum(
            Item.stat_bits[i] * (1 if mods[i] else 0)
            for i in range(0, 9)
        )

    def write(self):
        newpath = os.path.join(self.path, self.name)
        if self.fullpath != newpath:
            logging.info('[fm] renaming %s -> %s' % (self.fullpath, newpath))
            os.rename(self.fullpath, newpath)
        self.fullpath = os.path.join(self.path, self.name)
        os.chmod(self.fullpath, self.mode)

        err = None

        try:
            uid = int(self.owner or -1)
        except:
            try:
                uid = pwd.getpwnam(self.owner)[2]
            except KeyError:
                uid = -1
                err = Exception('Invalid owner')
        try:
            gid = int(self.group or -1)
        except:
            try:
                gid = grp.getgrnam(self.group)[2]
            except KeyError:
                gid = -1
                err = Exception('Invalid group')

        os.chown(self.fullpath, uid, gid)
        if err:
            raise err


@plugin
class FMBackend (BasePlugin):
    FG_OPERATION_LIMIT = 1024 * 1024 * 50

    def _escape(self, i):
        if hasattr(i, 'fullpath'):
            i = i.fullpath
        return '\'%s\' ' % i.replace("'", "\\'")

    def _total_size(self, items):
        return sum(_.size for _ in items)

    def _has_dirs(self, items):
        return any(_.isdir for _ in items)

    def init(self):
        self.task_manager = TaskManager.get()

    def remove(self, items, cb=lambda t: None):
        logging.info('[fm] removing %s' % ', '.join(x.fullpath for x in items))
        if self._total_size(items) > self.FG_OPERATION_LIMIT or self._has_dirs(items):
            paths = [x.fullpath for x in items]
            task = DeleteFilesTask.new(source=paths)
            task.callback = cb
            self.task_manager.run(task=task)
        else:
            for i in items:
                if os.path.isdir(i.fullpath):
                    shutil.rmtree(i.fullpath)
                else:
                    os.unlink(i.fullpath)
            cb(None)

    def move(self, items, dest, cb=lambda t: None):
        logging.info('[fm] moving %s to %s' % (', '.join(x.fullpath for x in items), dest))
        if self._total_size(items) > self.FG_OPERATION_LIMIT or self._has_dirs(items):
            paths = [x.fullpath for x in items]
            task = MoveFilesTask.new(source=paths, destination=dest)
            task.callback = cb
            self.task_manager.run(task=task)
        else:
            for i in items:
                shutil.move(i.fullpath, dest)
            cb(None)

    def copy(self, items, dest, cb=lambda t: None):
        logging.info('[fm] copying %s to %s' % (', '.join(x.fullpath for x in items), dest))
        if self._total_size(items) > self.FG_OPERATION_LIMIT or self._has_dirs(items):
            paths = [x.fullpath for x in items]
            task = CopyFilesTask.new(source=paths, destination=dest)
            task.callback = cb
            self.task_manager.run(task=task)
        else:
            for i in items:
                if os.path.isdir(i.fullpath):
                    shutil.copytree(i.fullpath, os.path.join(dest, i.name))
                else:
                    shutil.copy(i.fullpath, os.path.join(dest, i.name))
            cb(None)


@interface
class Unpacker (BasePlugin):
    @staticmethod
    def find(fn):
        for u in Unpacker.get_all():
            if u.match(fn):
                return u

    def match(self, fn):
        pass

    def unpack(self, path, cb=lambda: None):
        pass


@plugin
class TarUnpacker (Unpacker):
    def match(self, fn):
        return any(re.match(x, fn) for x in [r'.+\.tar.gz', r'.+\.tgz', r'.+\.tar'])

    def unpack(self, fn, cb=lambda: None):
        self.context.launch('terminal', command='cd "%s"; tar xvf "%s"' % os.path.split(fn), callback=cb)


@plugin
class ZipUnpacker (Unpacker):
    def match(self, fn):
        return any(re.match(x, fn) for x in [r'.+\.zip'])

    def unpack(self, fn, cb=lambda: None):
        self.context.launch('terminal', command='cd "%s"; unzip "%s"' % os.path.split(fn), callback=cb)


import subprocess

from ajenti.api import *
from api import PackageInfo, PackageManager


@plugin
@rootcontext
@persistent
class YumPackageManager (PackageManager):
    platforms = ['centos']

    def refresh(self):
        try:
            out_u = subprocess.check_output(['yum', '-C', '-q', '-d0', '-e0', 'check-update'])
        except subprocess.CalledProcessError as e:
            # yum check-update returns 100 if there are packages available for an update
            if e.returncode == 100:
                out_u = e.output
            else:
                raise e
        out_a = subprocess.check_output(['yum', '-C', '-d0', '-e0', 'list', 'installed', '-q'])
        self.all = self._parse_yum(out_a)
        self.all_dict = dict((x.name, x) for x in self.all)
        self.upgradeable = self._parse_yum(out_u)

    def search(self, query):
        out_s = subprocess.check_output(['yum', '-C', '-q', '-d0', '-e0', 'search', query])
        r = []
        for l in out_s.split('\n'):
            s = l.split()
            if len(s) < 2:
                continue
            if s[0].startswith('====') or s[0] == ':':
                continue
            else:
                p = PackageInfo()
                p.name = s[0]
                p.state = 'r'
                if p.name in self.all_dict and self.all_dict[p.name].state == 'i':
                    p.state = 'i'
                r.append(p)
        return r

    def get_lists(self):
        self.context.launch('terminal', command='yum check-update')

    def do(self, actions, callback=lambda: 0):
        to_install = [a for a in actions if a.action == 'i']
        to_remove = [a for a in actions if a.action == 'r']
        cmd = ''
        if len(to_install) > 0:
            cmd += 'yum install ' + ' '.join(a.name for a in to_install)
            if len(to_remove) > 0:
                cmd += ' && '
        if len(to_remove) > 0:
            cmd += 'yum remove ' + ' '.join(a.name for a in to_remove)
        self.context.launch('terminal', command=cmd, callback=callback)

    def _parse_yum(self, ss):
        r = []
        for s in ss.splitlines():
            s = s.split()
            try:
                if s[0] == '':
                    continue
                else:
                    p = PackageInfo()
                    p.name = s[0]
                    p.state = 'i'
                    r.append(p)
                if len(r.keys()) > 250:
                    break
            except:
                pass
        return r



import logging
import os
import pymongo
import subprocess
import tempfile

from ajenti.api import *
from ajenti.plugins.configurator.api import ClassConfigEditor
from ajenti.plugins.db_common.api import Database, User


@plugin
class MongoClassConfigEditor (ClassConfigEditor):
    title = 'MongoDB'
    icon = 'table'

    def init(self):
        self.append(self.ui.inflate('mongo:config'))


class MongoConnection (object):
    def __init__(self, db):
        self.client = pymongo.MongoClient(db.classconfig['url'])

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.client.disconnect()


@plugin
class MongoDB (BasePlugin):
    default_classconfig = {'url': 'mongodb://localhost:27017/'}
    classconfig_editor = MongoClassConfigEditor

    def connect(self):
        return MongoConnection(self)

    def query_sql(self, db, sql):
        f = tempfile.NamedTemporaryFile(suffix='.js', delete=False)
        f.write(sql)
        f.close()
            
        url = self.classconfig['url']
        if url.startswith('mongodb://'):
            url = url[10:]

        p = subprocess.Popen(
            [
                'mongo',
                url.rstrip('/') + '/' + db,    
                f.name,
            ],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        o, e = p.communicate(sql)
        os.unlink(f.name)
        if p.returncode:
            raise Exception(o + e)
        return filter(None, [[x] for x in (o + e).splitlines()])

    def query_databases(self):
        r = []
        with self.connect() as mongo:
            for x in mongo.client.database_names():
                d = Database()
                d.name = x
                r.append(d)
        return r

    def query_drop(self, db):
        with self.connect() as mongo:
            return mongo.client.drop_database(db.name)

    def query_create(self, name):
        with self.connect() as mongo:
            return mongo.client[name]['_tmp'].insert(0)



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Database Commons',
    icon='table',
    dependencies=[
        PluginDependency('main'),
        PluginDependency('services'),
    ],
)


def init():
    import api



import os
import subprocess

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.plugins.services.api import ServiceMultiplexor
from ajenti.ui import on
from ajenti.ui.binder import Binder
from ajenti.util import platform_select

from reconfigure.configs.base import Reconfig
from reconfigure.parsers import SSVParser
from reconfigure.builders import BoundBuilder
from reconfigure.nodes import Node, PropertyNode
from reconfigure.items.bound import BoundData


# SNMP

class SNMPData (BoundData):
    pass


class MIBData (BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value', 'mibs')),
            Node('token', PropertyNode('value', 'IF-MIB')),
        )


SNMPData.bind_collection('mibs', selector=lambda x: x.children[0].get('value').value == 'mibs', item_class=MIBData)
MIBData.bind_property('value', 'name', path=lambda x: x.children[1])


class SNMPConfig (Reconfig):
    def __init__(self, **kwargs):
        k = {
            'parser': SSVParser(),
            'builder': BoundBuilder(SNMPData),
        }
        k.update(kwargs)
        Reconfig.__init__(self, **k)


# SNMPD 

class SNMPDData (BoundData):
    pass


class ROCommunityData (BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value', 'rocommunity')),
            Node('token', PropertyNode('value', 'public 192.168.0.0/24')),
        )


class RWCommunityData (BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value', 'rwcommunity')),
            Node('token', PropertyNode('value', 'public 192.168.0.0/24')),
        )


class Sink1Data (BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value', 'trapsink')),
            Node('token', PropertyNode('value', 'localhost public')),
        )


class Sink2Data (BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value', 'trap2sink')),
            Node('token', PropertyNode('value', 'localhost public')),
        )


class Sink2cData (BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value', 'informsink')),
            Node('token', PropertyNode('value', 'localhost public')),
        )

SNMPDData.bind_collection('rocommunities', selector=lambda x: x.children[0].get('value').value == 'rocommunity', item_class=ROCommunityData)
SNMPDData.bind_collection('rwcommunities', selector=lambda x: x.children[0].get('value').value == 'rwcommunity', item_class=RWCommunityData)
SNMPDData.bind_collection('sinks1', selector=lambda x: x.children[0].get('value').value == 'trapsink', item_class=Sink1Data)
SNMPDData.bind_collection('sinks2', selector=lambda x: x.children[0].get('value').value == 'trap2sink', item_class=Sink2Data)
SNMPDData.bind_collection('sinks2c', selector=lambda x: x.children[0].get('value').value == 'informsink', item_class=Sink2cData)
ROCommunityData.bind_property('value', 'value', path=lambda x: x.children[1])
RWCommunityData.bind_property('value', 'value', path=lambda x: x.children[1])
for s in [Sink1Data, Sink2Data, Sink2cData]:
    s.bind_property('value', 'value', path=lambda x: x.children[1])


class SNMPDConfig (Reconfig):
    def __init__(self, **kwargs):
        k = {
            'parser': SSVParser(maxsplit=1),
            'builder': BoundBuilder(SNMPDData),
        }
        k.update(kwargs)
        Reconfig.__init__(self, **k)



@plugin
class SNMPDPlugin (SectionPlugin):
    service_name = platform_select(
        default='snmpd',
    )

    def init(self):
        self.title = 'SNMP'
        self.icon = 'exchange'
        self.category = _('Software')

        self.append(self.ui.inflate('snmpd:main'))

        self.find('servicebar').name = self.service_name
        self.find('servicebar').reload()

        self.snmp_config = SNMPConfig(path=platform_select(
            default='/etc/snmp/snmp.conf',
        ))
        self.snmpd_config = SNMPDConfig(path=platform_select(
            default='/etc/snmp/snmpd.conf',
        ))

        self.find('rocommunities').new_item = lambda c: ROCommunityData()
        self.find('rwcommunities').new_item = lambda c: RWCommunityData()
        self.find('sinks1').new_item = lambda c: Sink1Data()
        self.find('sinks2').new_item = lambda c: Sink2Data()
        self.find('sinks2c').new_item = lambda c: Sink2cData()

        self.binder = Binder(None, self)

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.snmp_config.load()
        self.snmpd_config.load()

        self.rocommunities = self.snmpd_config.tree.rocommunities
        self.rwcommunities = self.snmpd_config.tree.rwcommunities
        self.sinks1 = self.snmpd_config.tree.sinks1
        self.sinks2 = self.snmpd_config.tree.sinks2
        self.sinks2c = self.snmpd_config.tree.sinks2c

        enabled_mibs = []
        for mib in self.snmp_config.tree.mibs:
            for x in mib.name.strip('-+:').split(':'):
                enabled_mibs.append(x)
        self.mibs = []
        for dirpath, dirname, filenames in os.walk('/usr/share/mibs', followlinks=True):
            for x in filenames:
                if not x.startswith('.'):
                    mib = MIBData()
                    mib.name = x
                    mib.selected = x in enabled_mibs
                    self.mibs.append(mib)
        self.mibs = sorted(self.mibs, key=lambda x: x.name)
        self.binder.setup(self).populate()

    @on('save', 'click')
    def save(self):
        self.binder.update()
        
        mib = MIBData()
        mib.name = ':'.join([x.name for x in self.mibs if x.selected])
        for x in list(self.snmp_config.tree.mibs):
            self.snmp_config.tree.mibs.remove(x)
        self.snmp_config.tree.mibs.append(mib)

        self.snmp_config.save()
        self.snmpd_config.save()

        self.refresh()
        self.context.notify('info', _('Saved'))
        ServiceMultiplexor.get().get_one(self.service_name).restart()



from ajenti.api import *
from ajenti.plugins.db_common.api import DBPlugin
from ajenti.util import platform_select

from api import MySQLDB


@plugin
class MySQLPlugin (DBPlugin):
    config_class = MySQLDB

    service_name = platform_select(
        debian='mysql',
        default='mysqld'
    )

    service_buttons = [
        {
            'command': 'reload',
            'text': _('Reload'),
            'icon': 'step-forward',
        }
    ]

    def init(self):
        self.title = 'MySQL'
        self.category = _('Software')
        self.icon = 'table'
        self.db = MySQLDB.get()

    def query_sql(self, db, sql):
        return self.db.query_sql(db, sql)

    def query_databases(self):
        return self.db.query_databases()

    def query_drop(self, db):
        return self.db.query_drop(db)

    def query_create(self, name):
        return self.db.query_create(name)

    def query_users(self):
        return self.db.query_users()

    def query_create_user(self, user):
        return self.db.query_create_user(user)

    def query_drop_user(self, user):
        return self.db.query_drop_user(user)

    def query_grant(self, user, db):
        return self.db.query_grant(user, db)



import os
import subprocess

from ajenti.api import *
from ajenti.util import cache_value

from api import Service, ServiceManager


@plugin
class OSXServiceManager (ServiceManager):
    platforms = ['osx']

    @cache_value(1)
    def get_all(self):
        r = []
        for line in subprocess.check_output(['launchctl', 'list']).splitlines()[1:]:
            tokens = line.split()
            if len(tokens) == 3:
                s = OSXService(tokens[2])
                if not '.anonymous.' in s.name:
                    s.running = tokens[0] != '-'
                    r.append(s)
        return r


class OSXService (Service):
    source = 'launchd'

    def __init__(self, name):
        self.name = name

    def refresh(self):
        self.running = OSXServiceManager().get_one(self.name).running

    def start(self):
        self.command('start')

    def stop(self):
        self.command('stop')

    def restart(self):
        self.command('stop')
        self.command('start')

    def command(self, cmd):
        subprocess.call(['launchctl', cmd, self.name])



import os
import subprocess

from ajenti.api import *
from ajenti.api.helpers import subprocess_call_background, subprocess_check_output_background
from ajenti.util import cache_value

from api import ServiceManager
from sm_sysvinit import SysVInitService


@plugin
class CentOSServiceManager (ServiceManager):
    platforms = ['centos']

    @cache_value(1)
    def get_all(self):
        r = []
        pending = {}
        for line in subprocess_check_output_background(['chkconfig', '--list']).splitlines():
            tokens = line.split()
            if len(tokens) < 3:
                continue

            name = tokens[0]
            s = SysVInitService(name)
            s.refresh()
            r.append(s)

        return r

    def get_one(self, name):
        s = SysVInitService(name)
        if os.path.exists(s.script):
            s.refresh()
            return s
        return None



from ajenti.api import *
from ajenti.plugins import *
from ajenti.util import platform_select


info = PluginInfo(
    title='Apache',
    description='Apache 2 web server',
    icon='globe',
    dependencies=[
        PluginDependency('webserver_common'),
        BinaryDependency(platform_select(
            centos='httpd',
            freebsd='apachectl',
            default='apache2',
        )),
    ],
)


def init():
    import main



from ajenti.api import *


@interface
class UserSyncProvider (BasePlugin):
    allows_renaming = False
    syncs_root = False
    
    def test(self):
        return False

    def check_password(self, username, password):
        return False

    def sync(self):
        pass


import gevent
import json
import logging
import os
import requests

from ajenti.api import *
from ajenti.ipc import IPCHandler
from ajenti.plugins import manager
from ajenti.util import *


HOST = 'ajenti.org'
URL = 'http://%s/licensing-api/' % HOST
LICENSE_PATH = '/var/lib/ajenti/license'


@public
@plugin
@persistent
@rootcontext
class Licensing (BasePlugin):
    licensing_active = True

    def init(self):
        gevent.spawn(self.worker)
        self.__license_status = {}

    def get_license_status(self):
        return self.__license_status

    def read_license(self):
        if not os.path.exists(LICENSE_PATH):
            return None
        return open(LICENSE_PATH).read()

    def remove_license(self):
        if os.path.exists(LICENSE_PATH):
            os.unlink(LICENSE_PATH)
        self.__license_status = {}

    def write_license(self, key):
        open(LICENSE_PATH, 'w').write(key)
        os.chmod(LICENSE_PATH, 0600)

    def activate(self):
        response = requests.post(URL + 'activate?key=' + self.read_license())
        logging.debug('Licensing << %s' % response.content)
        response = json.loads(response.content)
        if response['status'] == 'ok' and self.__license_status == {}:
            logging.info('License activated')
        self.__license_status = response
        return self.__license_status

    def deactivate(self):
        response = requests.post(URL + 'deactivate?key=' + self.read_license())
        logging.debug('Licensing << %s' % response.content)

    def worker(self):
        while True:
            try:
                if self.read_license() is not None:
                    self.activate()
            except:
                pass
            gevent.sleep(3600 * 12)


@plugin
class LicensingIPC (IPCHandler):
    def init(self):
        self.manager = Licensing.get()

    def get_name(self):
        return 'license'

    def handle(self, args):
        command = args[0]
        if command == 'install':
            if len(args) != 2:
                return 'ajenti-ipc license install <key>'
            self.manager.write_license(args[1])
            return json.dumps(self.manager.activate())
        if command == 'remove':
            self.manager.deactivate()
            self.manager.remove_license()
            return 'OK'



import os
import subprocess


class SambaMonitor (object):
    def __init__(self):
        self.refresh()

    def refresh(self):
        pids = {}

        ll = subprocess.check_output(['smbstatus', '-p']).splitlines()
        for l in ll[4:]:
            s = l.split()
            if len(s) > 0:
                pids[s[0]] = (s[1], ' '.join(s[3:]))

        self.connections = []
        ll = subprocess.check_output(['smbstatus', '-S']).splitlines()
        for l in ll[3:]:
            s = l.split()
            if len(s) > 0 and s[1] in pids:
                c = SambaConnection(s[0], s[1], *pids[s[1]])
                self.connections.append(c)


class SambaConnection (object):
    def __init__(self, share, pid, user, machine):
        self.share, self.pid, self.user, self.machine = share, pid, user, machine

    def disconnect(self):
        os.kill(int(self.pid), 15)



from datetime import datetime
import time
import subprocess

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.plugins.services.api import ServiceMultiplexor
from ajenti.ui import on
from ajenti.ui.binder import Binder
from ajenti.util import platform_select

from reconfigure.configs.base import Reconfig
from reconfigure.parsers import SSVParser
from reconfigure.builders import BoundBuilder
from reconfigure.nodes import Node, PropertyNode
from reconfigure.items.bound import BoundData


class NTPDData (BoundData):
    pass


class ServerData (BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value',  'server')),
            Node('token', PropertyNode('value', '0.pool.ntp.org')),
        )


NTPDData.bind_collection('servers', selector=lambda x: x.children[0].get('value').value == 'server', item_class=ServerData)
ServerData.bind_property('value', 'address', path=lambda x: x.children[1])


class NTPDConfig (Reconfig):
    def __init__(self, **kwargs):
        k = {
            'parser': SSVParser(),
            'builder': BoundBuilder(NTPDData),
        }
        k.update(kwargs)
        Reconfig.__init__(self, **k)


@plugin
class NTPDPlugin (SectionPlugin):
    service_name = platform_select(
        default='ntp',
        centos='ntpd',
    )

    def init(self):
        self.title = _('Date & Time')
        self.icon = 'time'
        self.category = _('Software')

        self.append(self.ui.inflate('ntpd:main'))

        self.find('servicebar').name = self.service_name
        self.find('servicebar').reload()

        self.config = NTPDConfig(path=platform_select(
            default='/etc/ntp.conf',
        ))

        self.binder = Binder(None, self)

        self.find('servers').new_item = lambda c: ServerData()

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.config.load()
        self.now = int(time.time())
        self.binder.setup(self).populate()

    @on('set', 'click')
    def on_set(self):
        self.binder.update()
        d = datetime.fromtimestamp(self.now)
        s = d.strftime('%m%d%H%M%Y') 
        subprocess.call(['date', s])
        self.refresh()

    @on('sync', 'click')
    def on_sync(self):
        self.binder.update()
        if len(self.config.tree.servers) == 0:
            self.context.notify('error', _('No servers defined'))
            return
        server = self.config.tree.servers[0].address
        output = subprocess.check_output(['ntpdate', '-u', server])
        self.context.notify('info', _('Done'))
        self.context.notify('info', output)
        self.refresh()

    @on('save', 'click')
    def save(self):
        self.binder.update()
        self.config.save()
        self.refresh()
        self.context.notify('info', _('Saved'))
        ServiceMultiplexor.get().get_one(self.service_name).restart()



import dbus
import os

import subprocess
import logging

from ajenti.api import *
from ajenti.util import cache_value

from api import Service, ServiceManager


@plugin
class SystemdServiceManager (ServiceManager):
    def init(self):
        self.bus = dbus.SystemBus()
        self.systemd = self.bus.get_object("org.freedesktop.systemd1", "/org/freedesktop/systemd1")
        self.interface = dbus.Interface(self.systemd, 'org.freedesktop.systemd1.Manager')

    @classmethod
    def verify(cls):
        return subprocess.call(['which', 'systemctl']) == 0

    @cache_value(1)
    def get_all(self):
        unit_files = self.interface.ListUnitFiles() 
        r = []
        for unit_file in unit_files:
            if str(unit_file[0]).endswith('.service'):
                name = str(unit_file[0])
                name = os.path.splitext(name)[0]
                name = os.path.split(name)[1]
                r.append(SystemdService(name))

        units = self.interface.ListUnits()
        for unit in units:
            name = str(unit[0])
            name = os.path.splitext(name)[0]
            for service in r:
                if service.name == name:
                    service.running = str(unit[4]) == 'running'

        return r


class SystemdService (Service):
    source = 'systemd'

    def __init__(self, name):
        self.name = name
        self.running = False

    def refresh(self):
        self.running = subprocess.call(['systemctl', 'is-active', self.name]) == 0

    def start(self):
        self.command('start')

    def stop(self):
        self.command('stop')

    def restart(self):
        self.command('restart')

    def command(self, cmd):
        return subprocess.call(['systemctl', cmd, self.name], close_fds=True)



import gettext
import os

import ajenti


language = ajenti.config.tree.language
localedir = os.path.abspath(os.path.join(os.path.split(ajenti.__file__)[0], 'locales'))
try:
    translation = gettext.translation(
        'ajenti', 
        localedir=localedir, 
        languages=([language] if language else None),
    )
except IOError:  # not such translation
    translation = gettext.NullTranslations()
translation.install(unicode=True)  


def list_locales():
    return [_ for _ in os.listdir(localedir) if len(_) == 5]



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='fail2ban',
    icon='shield',
    dependencies=[
        PluginDependency('main'),
        BinaryDependency('fail2ban-client'),
        BinaryDependency('fail2ban-regex'),
    ],
)


def init():
    import main


# -*- coding: utf-8 -*-
"""
    pyte.graphics
    ~~~~~~~~~~~~~

    This module defines graphic-related constants, mostly taken from
    :manpage:`console_codes(4)` and
    http://pueblo.sourceforge.net/doc/manual/ansi_color_codes.html.

    :copyright: (c) 2011-2013 by Selectel, see AUTHORS for details.
    :license: LGPL, see LICENSE for more details.
"""

import itertools


#: A mapping of ANSI text style codes to style names, "+" means the:
#: attribute is set, "-" -- reset; example:
#:
#: >>> text[1]
#: '+bold'
#: >>> text[9]
#: '+strikethrough'
TEXT = {
    1: "+bold" ,
    3: "+italics",
    4: "+underscore",
    7: "+reverse",
    9: "+strikethrough",
    22: "-bold",
    23: "-italics",
    24: "-underscore",
    27: "-reverse",
    29: "-strikethrough"
}


#: A mapping of ANSI foreground color codes to color names, example:
#:
#: >>> FG[30]
#: 'black'
#: >>> FG[38]
#: 'default'
FG = {
    30: "black",
    31: "red",
    32: "green",
    33: "brown",
    34: "blue",
    35: "magenta",
    36: "cyan",
    37: "white",
    39: "default"  # white.
}

#: A mapping of ANSI background color codes to color names, example:
#:
#: >>> BG[40]
#: 'black'
#: >>> BG[48]
#: 'default'
BG = {
    40: "black",
    41: "red",
    42: "green",
    43: "brown",
    44: "blue",
    45: "magenta",
    46: "cyan",
    47: "white",
    49: "default"  # black.
}

# Reverse mapping of all available attributes -- keep this private!
_SGR = dict((v, k) for k, v in itertools.chain(BG.items(),
                                               FG.items(),
                                               TEXT.items()))



from ajenti.api import *
from ajenti.plugins.db_common.api import DBPlugin
from ajenti.util import platform_select

from api import MongoDB


@plugin
class MongoDBPlugin (DBPlugin):
    config_class = MongoDB
    has_users = False

    service_name = platform_select(
        default='mongod'
    )

    def init(self):
        self.title = 'MongoDB'
        self.category = _('Software')
        self.icon = 'table'
        self.db = MongoDB.get()

    def query_sql(self, db, sql):
        return self.db.query_sql(db, sql)

    def query_databases(self):
        return self.db.query_databases()

    def query_drop(self, db):
        return self.db.query_drop(db)

    def query_create(self, name):
        return self.db.query_create(name)



from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.plugins import manager
from ajenti.ui import on
from ajenti.ui.binder import Binder, DictAutoBinding

from api import Task, TaskDefinition, JobDefinition
from manager import TaskManager


@plugin
class Tasks (SectionPlugin):
    def init(self):
        self.title = _('Tasks')
        self.icon = 'cog'
        self.category = _('Tools')

        self.append(self.ui.inflate('tasks:main'))

        self.manager = TaskManager.get()
        self.binder = Binder(None, self)

        def post_td_bind(object, collection, item, ui):
            if item.get_class():
                params_ui = self.ui.inflate(item.get_class().ui)
                item.binder = DictAutoBinding(item, 'params', params_ui.find('bind'))
                item.binder.populate()
                ui.find('slot').empty()
                ui.find('slot').append(params_ui)

        def post_td_update(object, collection, item, ui):
            if hasattr(item, 'binder'):
                item.binder.update()

        def post_rt_bind(object, collection, item, ui):
            def abort():
                item.abort()
                self.refresh()
            ui.find('abort').on('click', abort)

        self.find('task_definitions').post_item_bind = post_td_bind
        self.find('task_definitions').post_item_update = post_td_update
        self.find('running_tasks').post_item_bind = post_rt_bind

        self.find('job_definitions').new_item = lambda c: JobDefinition()

    def on_page_load(self):
        self.refresh()

    @on('refresh', 'click')
    def refresh(self):
        self.manager.refresh()

        self.binder.unpopulate()

        dd = self.find('task-classes')
        dd.labels = []
        dd.values = []
        for task in Task.get_classes():
            if not task.hidden:
                dd.labels.append(task.name)
                dd.values.append(task.classname)

        dd = self.find('task-selector')
        dd.labels = [_.name for _ in self.manager.task_definitions]
        dd.values = [_.id for _ in self.manager.task_definitions]
        self.find('run-task-selector').labels = dd.labels
        self.find('run-task-selector').values = dd.values

        self.binder.setup(self.manager).populate()

    @on('run-task', 'click')
    def on_run_task(self):
        self.manager.run(task_id=self.find('run-task-selector').value, context=self.context)
        self.refresh()

    @on('create-task', 'click')
    def on_create_task(self):
        cls = self.find('task-classes').value
        td = TaskDefinition(task_class=cls)
        td.name = td.get_class().name

        self.manager.task_definitions.append(td)
        self.refresh()

    @on('save', 'click')
    def on_save(self):
        self.binder.update()
        self.manager.save()
        self.refresh()



import gevent
import subprocess

from ajenti.api import plugin
from ajenti.plugins.dashboard.api import ConfigurableWidget
from ajenti.users import PermissionProvider, restrict
from ajenti.ui import on


@plugin
class ScriptWidget (ConfigurableWidget):
    name = _('Script')
    icon = 'play'

    def on_prepare(self):
        self.append(self.ui.inflate('scripts:widget'))

    def on_start(self):
        self.command = self.config['command']
        if not self.command:
            return
        self.find('name').text = self.config['title']

    def create_config(self):
        return {'command': '', 'title': '', 'terminal': False}

    def on_config_start(self):
        self.dialog.find('command').value = self.config['command']
        self.dialog.find('title').value = self.config['title']
        self.dialog.find('terminal').value = self.config['terminal']

    @on('edit', 'click')
    def on_edit(self):
        self.begin_configuration()

    def on_config_save(self):
        self.config['command'] = self.dialog.find('command').value
        self.config['title'] = self.dialog.find('title').value
        self.config['terminal'] = self.dialog.find('terminal').value

    @on('start', 'click')
    @restrict('scripts:run')
    def on_s_start(self):
        if self.config['terminal']:
            self.context.launch('terminal', command=self.config['command'])
        else:
            p = subprocess.Popen(self.config['command'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            self.context.notify('info', _('Launched'))

            def worker():
                o, e = p.communicate()
                self.context.notify('info', o + e)

            gevent.spawn(worker)


@plugin
class ScriptPermissionsProvider (PermissionProvider):
    def get_name(self):
        return _('Scripts')

    def get_permissions(self):
        return [
            ('scripts:run', _('Run scripts')),
        ]



import logging
import logging.handlers
import os
import sys
from datetime import datetime

from ajenti.api import extract_context


LOG_DIR = '/var/log/ajenti'
LOG_NAME = 'ajenti.log'
LOG_FILE = os.path.join(LOG_DIR, LOG_NAME)


class DebugHandler (logging.StreamHandler):
    """
    Captures log into a buffer for error reports
    """

    def __init__(self):
        self.capturing = False
        self.buffer = ''

    def start(self):
        self.capturing = True

    def stop(self):
        self.capturing = False

    def handle(self, record):
        if self.capturing:
            self.buffer += self.formatter.format(record) + '\n'


class ConsoleHandler (logging.StreamHandler):
    def __init__(self, stream, debug):
        self.debug = debug
        logging.StreamHandler.__init__(self, stream)

    def handle(self, record):
        if not self.stream.isatty():
            return logging.StreamHandler.handle(self, record)

        s = ''
        d = datetime.fromtimestamp(record.created)
        s += d.strftime("\033[37m%d.%m.%Y %H:%M \033[0m")
        if self.debug:
            s += ('%s:%s' % (record.filename, record.lineno)).ljust(30)
        l = ''
        if record.levelname == 'DEBUG':
            l = '\033[37mDEBUG\033[0m '
        if record.levelname == 'INFO':
            l = '\033[32mINFO\033[0m  '
        if record.levelname == 'WARNING':
            l = '\033[33mWARN\033[0m  '
        if record.levelname == 'ERROR':
            l = '\033[31mERROR\033[0m '
        s += l.ljust(9)

        context = extract_context()
        if hasattr(context, 'session') and hasattr(context.session, 'identity'):
            s += '[%s] ' % context.session.identity

        try:
            s += record.msg % record.args
        except:
            s += record.msg
        s += '\n'
        self.stream.write(s)


def make_log(debug=False, log_level=logging.INFO):
    log = logging.getLogger()
    log.setLevel(logging.DEBUG)

    stdout = ConsoleHandler(sys.stdout, debug)
    stdout.setLevel(log_level)

    logging.blackbox = DebugHandler()
    logging.blackbox.setLevel(logging.DEBUG)
    dformatter = logging.Formatter('%(asctime)s %(levelname)-8s %(module)s.%(funcName)s(): %(message)s')
    logging.blackbox.setFormatter(dformatter)
    stdout.setFormatter(dformatter)
    log.addHandler(logging.blackbox)

    log.addHandler(stdout)


def init_log_directory():
    if not os.path.exists(LOG_DIR):
        os.mkdir(LOG_DIR)


def init_log_rotation():
    log = logging.getLogger()
    try:
        handler = logging.handlers.TimedRotatingFileHandler(
            os.path.join(LOG_DIR, LOG_NAME),
            when='midnight',
            backupCount=7
        )
        handler.setLevel(logging.INFO)
        handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-8s: %(message)s'))
        log.addHandler(handler)
    except IOError:
        pass

    return log


def init(level=logging.INFO):
    make_log(debug=level == logging.DEBUG, log_level=level)
    logging.blackbox.start()



from ajenti.api import *
from ajenti.ui.binder import Binder
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.util import platform_select

from reconfigure.configs import SupervisorConfig
from reconfigure.items.supervisor import ProgramData

from client import SupervisorServiceManager


@plugin
class Supervisor (SectionPlugin):
    def init(self):
        self.title = 'Supervisor'
        self.icon = 'play'
        self.category = _('Software')
        self.append(self.ui.inflate('supervisor:main'))
        self.mgr = SupervisorServiceManager.get()
        self.binder = Binder(None, self.find('main'))
        self.find('programs').new_item = lambda c: ProgramData()
        self.config = SupervisorConfig(path=platform_select(
            default='/etc/supervisor/supervisord.conf',
            centos='/etc/supervisord.conf',
        ))
        self.find('servicebar').name = platform_select(
            centos='supervisord',
            default='supervisor',
        )
        self.find('servicebar').reload()

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.config.load()
        self.mgr.fill(self.config.tree.programs)
        self.binder.setup(self.config.tree).populate()

    @on('save', 'click')
    def on_save(self):
        self.binder.update()
        self.config.save()
        self.refresh()



import subprocess
import logging

from ajenti.api import *
from api import PackageInfo, PackageManager


@plugin
@rootcontext
@persistent
class ArchPackageManager (PackageManager):
    platforms = ['arch']

    def init(self):
        self.upgradeable = []
        self.all = []

    def get_lists(self):
        self.context.launch('terminal', command='pacman -Sy')

    def refresh(self):
        try:
            out_u = subprocess.check_output(['pacman', '-Qu'])
        except subprocess.CalledProcessError as cpe:
            out_u = ''

        out_a = subprocess.check_output(['pacman', '-Qs'])

        self.upgradeable = self._parse_upgradeable(out_u)

        self.all = self._parse_all_installed(out_a)
        self.all_dict = dict((x.name, x) for x in self.all)

    def search(self, query):
        try:
            out_s = subprocess.check_output(['pacman', '-Ss', query])
        except subprocess.CalledProcessError as cpe:
            self.context.notify('info', _('No search results found'))
            logging.info('No search results found')
            out_s = ''

        r = []
        for l in out_s.split('\n'):
            s = l.split('/')
            if len(s) == 2 and not(s[0].startswith('  ')):
                sn = s[1].split(' ')

                p = PackageInfo()
                p.name = sn[0]
                p.state = 'i' if subprocess.call(['pacman', '-Q', p.name]) == 0 else 'r'
                p.version = sn[1]

                r.append(p)

        return r

    def do(self, actions, callback=lambda: 0):
        to_install = [a for a in actions if a.action == 'i']
        to_remove = [a for a in actions if a.action == 'r']
        cmd = ''
        if len(to_install) > 0:
            cmd += 'pacman -S --noconfirm ' + ' '.join(a.name for a in to_install)
            if len(to_remove) > 0:
                cmd += ' && '

        if len(to_remove) > 0:
            cmd += 'pacman -R --noconfirm ' + ' '.join(a.name for a in to_remove)

        logging.debug('launching terminal with command %s' % cmd)
        self.context.launch('terminal', command=cmd, callback=callback)


    def _parse_upgradeable(self, d):
        r = []
        for l in d.split('\n'):
            s = l.split(' ')
            if len(s) == 1:
                continue

            p = PackageInfo()
            p.action = 'i'

            p.name = s[0]
            p.version = s[1]
            r.append(p)
        return r

    def _parse_all_installed(self, d):
        r = []

        lines = d.splitlines()
        infos = ['\n'.join(lines[i:i+2]) for i in range(0, len(lines), 2)]

        for info in infos:
            s = info.split('\n')
            if len(s) == 0:
                continue

            package = s[0].split(' ')

            p = PackageInfo()
            p.state = 'i'

            p.name = package[0]
            p.version = package[1]

            s[1].lstrip().rstrip()
            p.description = ' '.join(s[1])

            r.append(p)
        return r




import subprocess

from ajenti.api import *
from ajenti.api.sensors import Sensor
from ajenti.util import cache_value


@plugin
class IPMISensor (Sensor):
    id = 'ipmi'
    timeout = 5

    @cache_value(2)
    def _get_data(self):
        r = {}
        for l in subprocess.check_output(['ipmitool', 'sensor']).splitlines():
            l = l.split('|')
            if len(l) > 2:
                r[l[0].strip()] = (l[1].strip(), l[2].strip())
        return r

    def get_variants(self):
        return sorted(self._get_data().keys())

    def measure(self, variant):
        try:
            return self._get_data()[variant]
        except:
            return (0, '')



# coding=utf-8

config_templates = {
    'main': '',
    'jails': 'enable = true\n',
    'actions': """
# Fail2Ban configuration file
#
# Author:
#
#

[Definition]

# Option:  actionstart
# Notes.:  command executed once at the start of Fail2Ban.
# Values:  CMD
#
actionstart =


# Option:  actionstop
# Notes.:  command executed once at the end of Fail2Ban
# Values:  CMD
#
actionstop =


# Option:  actioncheck
# Notes.:  command executed once before each actionban command
# Values:  CMD
#
actioncheck =


# Option:  actionban
# Notes.:  command executed when banning an IP. Take care that the
#          command is executed with Fail2Ban user rights.
# Tags:    See jail.conf(5) man page
# Values:  CMD
#
actionban =


# Option:  actionunban
# Notes.:  command executed when unbanning an IP. Take care that the
#          command is executed with Fail2Ban user rights.
# Tags:    See jail.conf(5) man page
# Values:  CMD
#
actionunban =

[Init]

# Option:  port
# Notes.:  specifies port to monitor
# Values:  [ NUM | STRING ]
#
port =

# Option:  localhost
# Notes.:  the local IP address of the network interface
# Values:  IP
#
localhost = 127.0.0.1


# Option:  blocktype
# Notes.:  How to block the traffic. Use a action from man 5 ipfw
#          Common values: deny, unreach port, reset
# Values:  STRING
#
blocktype = unreach port

""",
    'filters': '# Fail2Ban filter\n[INCLUDES]\nbefore =\nafter =\n[Definition]\nfailregex =\n\nignoreregex =\n#Autor:',
    'extra': ''
}


# -*- coding: utf-8 -*-
"""
    pyte.streams
    ~~~~~~~~~~~~

    This module provides three stream implementations with different
    features; for starters, here's a quick example of how streams are
    typically used:

    >>> import pyte
    >>>
    >>> class Dummy(object):
    ...     def __init__(self):
    ...         self.y = 0
    ...
    ...     def cursor_up(self, count=None):
    ...         self.y += count or 1
    ...
    >>> dummy = Dummy()
    >>> stream = pyte.Stream()
    >>> stream.attach(dummy)
    >>> stream.feed(u"\u001B[5A")  # Move the cursor up 5 rows.
    >>> dummy.y
    5

    :copyright: (c) 2011-2013 by Selectel, see AUTHORS for details.
    :license: LGPL, see LICENSE for more details.
"""

from __future__ import absolute_import, unicode_literals

import os
import codecs
import sys

from . import control as ctrl, escape as esc


if sys.version_info[0] == 2:
    str = unicode


class Stream(object):
    """A stream is a state machine that parses a stream of characters
    and dispatches events based on what it sees.

    .. note::

       Stream only accepts  strings as input, but if, for some reason,
       you need to feed it with bytes, consider using
       :class:`~pyte.streams.ByteStream` instead.

    .. seealso::

        `man console_codes <http://linux.die.net/man/4/console_codes>`_
            For details on console codes listed bellow in :attr:`basic`,
            :attr:`escape`, :attr:`csi` and :attr:`sharp`.
    """

    #: Control sequences, which don't require any arguments.
    basic = {
        ctrl.BEL: "bell",
        ctrl.BS: "backspace",
        ctrl.HT: "tab",
        ctrl.LF: "linefeed",
        ctrl.VT: "linefeed",
        ctrl.FF: "linefeed",
        ctrl.CR: "carriage_return",
        ctrl.SO: "shift_out",
        ctrl.SI: "shift_in",
    }

    #: non-CSI escape sequences.
    escape = {
        esc.RIS: "reset",
        esc.IND: "index",
        esc.NEL: "linefeed",
        esc.RI: "reverse_index",
        esc.HTS: "set_tab_stop",
        esc.DECSC: "save_cursor",
        esc.DECRC: "restore_cursor",
    }

    #: "sharp" escape sequences -- ``ESC # <N>``.
    sharp = {
        esc.DECALN: "alignment_display",
    }

    #: CSI escape sequences -- ``CSI P1;P2;...;Pn <fn>``.
    csi = {
        esc.ICH: "insert_characters",
        esc.CUU: "cursor_up",
        esc.CUD: "cursor_down",
        esc.CUF: "cursor_forward",
        esc.CUB: "cursor_back",
        esc.CNL: "cursor_down1",
        esc.CPL: "cursor_up1",
        esc.CHA: "cursor_to_column",
        esc.CUP: "cursor_position",
        esc.ED: "erase_in_display",
        esc.EL: "erase_in_line",
        esc.IL: "insert_lines",
        esc.DL: "delete_lines",
        esc.DCH: "delete_characters",
        esc.ECH: "erase_characters",
        esc.HPR: "cursor_forward",
        esc.VPA: "cursor_to_line",
        esc.VPR: "cursor_down",
        esc.HVP: "cursor_position",
        esc.TBC: "clear_tab_stop",
        esc.SM: "set_mode",
        esc.RM: "reset_mode",
        esc.SGR: "select_graphic_rendition",
        esc.DECSTBM: "set_margins",
        esc.HPA: "cursor_to_column",
    }

    def __init__(self):
        self.handlers = {
            "stream": self._stream,
            "escape": self._escape,
            "arguments": self._arguments,
            "sharp": self._sharp,
            "charset": self._charset
        }

        self.listeners = []
        self.reset()

    def reset(self):
        """Reset state to ``"stream"`` and empty parameter attributes."""
        self.state = "stream"
        self.flags = {}
        self.params = []
        self.current = ""

    def consume(self, char):
        """Consume a single string character and advance the state as
        necessary.

        :param str char: a character to consume.
        """
        if not isinstance(char, str):
            raise TypeError("%s requires str input" % self.__class__.__name__)

        try:
            self.handlers.get(self.state)(char)
        except TypeError:
            pass
        except KeyError:
            if __debug__:
                self.flags["state"] = self.state
                self.flags["unhandled"] = char
                self.dispatch("debug", *self.params)
                self.reset()
            else:
                raise

    def feed(self, chars):
        """Consume a string and advance the state as necessary.

        :param str chars: a string to feed from.
        """
        if not isinstance(chars, str):
            raise TypeError("%s requires str input" % self.__class__.__name__)

        for char in chars: self.consume(char)

    def attach(self, screen, only=()):
        """Adds a given screen to the listeners queue.

        :param pyte.screens.Screen screen: a screen to attach to.
        :param list only: a list of events you want to dispatch to a
                          given screen (empty by default, which means
                          -- dispatch all events).
        """
        self.listeners.append((screen, set(only)))

    def detach(self, screen):
        """Removes a given screen from the listeners queue and failes
        silently if it's not attached.

        :param pyte.screens.Screen screen: a screen to detach.
        """
        for idx, (listener, _) in enumerate(self.listeners):
            if screen is listener:
                self.listeners.pop(idx)

    def dispatch(self, event, *args, **kwargs):
        """Dispatch an event.

        Event handlers are looked up implicitly in the listeners'
        ``__dict__``, so, if a listener only wants to handle ``DRAW``
        events it should define a ``draw()`` method or pass
        ``only=["draw"]`` argument to :meth:`attach`.

        .. warning::

           If any of the attached listeners throws an exception, the
           subsequent callbacks are be aborted.

        :param str event: event to dispatch.
        :param list args: arguments to pass to event handlers.
        """
        for listener, only in self.listeners:
            if only and event not in only:
                continue

            try:
                handler = getattr(listener, event)
            except AttributeError:
                continue

            if hasattr(listener, "__before__"):
                listener.__before__(event)

            handler(*args, **self.flags)

            if hasattr(listener, "__after__"):
                listener.__after__(event)
        else:
            if kwargs.get("reset", True): self.reset()

    # State transformers.
    # ...................

    def _stream(self, char):
        """Process a character when in the default ``"stream"`` state."""
        if char in self.basic:
            self.dispatch(self.basic[char])
        elif char == ctrl.ESC:
            self.state = "escape"
        elif char == ctrl.CSI:
            self.state = "arguments"
        elif char not in [ctrl.NUL, ctrl.DEL]:
            self.dispatch("draw", char)

    def _escape(self, char):
        """Handle characters seen when in an escape sequence.

        Most non-VT52 commands start with a left-bracket after the
        escape and then a stream of parameters and a command; with
        a single notable exception -- :data:`escape.DECOM` sequence,
        which starts with a sharp.
        """
        if char == "#":
            self.state = "sharp"
        elif char == "[":
            self.state = "arguments"
        elif char in "()":
            self.state = "charset"
            self.flags["mode"] = char
        else:
            self.dispatch(self.escape[char])

    def _sharp(self, char):
        """Parse arguments of a `"#"` seqence."""
        self.dispatch(self.sharp[char])

    def _charset(self, char):
        """Parse ``G0`` or ``G1`` charset code."""
        self.dispatch("set_charset", char)

    def _arguments(self, char):
        """Parse arguments of an escape sequence.

        All parameters are unsigned, positive decimal integers, with
        the most significant digit sent first. Any parameter greater
        than 9999 is set to 9999. If you do not specify a value, a 0
        value is assumed.

        .. seealso::

           `VT102 User Guide <http://vt100.net/docs/vt102-ug/>`_
               For details on the formatting of escape arguments.

           `VT220 Programmer Reference <http://http://vt100.net/docs/vt220-rm/>`_
               For details on the characters valid for use as arguments.
        """
        if char == "?":
            self.flags["private"] = True
        elif char in [ctrl.BEL, ctrl.BS, ctrl.HT, ctrl.LF, ctrl.VT,
                      ctrl.FF, ctrl.CR]:
            # Not sure why, but those seem to be allowed between CSI
            # sequence arguments.
            self.dispatch(self.basic[char], reset=False)
        elif char == ctrl.SP:
            pass
        elif char in [ctrl.CAN, ctrl.SUB]:
            # If CAN or SUB is received during a sequence, the current
            # sequence is aborted; terminal displays the substitute
            # character, followed by characters in the sequence received
            # after CAN or SUB.
            self.dispatch("draw", char)
            self.state = "stream"
        elif char.isdigit():
            self.current += char
        else:
            self.params.append(min(int(self.current or 0), 9999))

            if char == ";":
                self.current = ""
            else:
                self.dispatch(self.csi[char], *self.params)


class ByteStream(Stream):
    """A stream, which takes bytes (instead of strings) as input
    and tries to decode them using a given list of possible encodings.
    It uses :class:`codecs.IncrementalDecoder` internally, so broken
    bytes is not an issue.

    By default, the following decoding strategy is used:

    * First, try strict ``"utf-8"``, proceed if recieved and
      :exc:`UnicodeDecodeError` ...
    * Try strict ``"cp437"``, failed? move on ...
    * Use ``"utf-8"`` with invalid bytes replaced -- this one will
      allways succeed.

    >>> stream = ByteStream()
    >>> stream.feed(b"foo".decode("utf-8"))
    Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
      File "pyte/streams.py", line 323, in feed
        "%s requires input in bytes" % self.__class__.__name__)
    TypeError: ByteStream requires input in bytes
    >>> stream.feed(b"foo")

    :param list encodings: a list of ``(encoding, errors)`` pairs,
                           where the first element is encoding name,
                           ex: ``"utf-8"`` and second defines how
                           decoding errors should be handeld; see
                           :meth:`str.decode` for possible values.
    """

    def __init__(self, encodings=None):
        encodings = encodings or [
            ("utf-8", "strict"),
            ("cp437", "strict"),
            ("utf-8", "replace")
        ]

        self.buffer = b"", 0
        self.decoders = [codecs.getincrementaldecoder(encoding)(errors)
                         for encoding, errors in encodings]

        super(ByteStream, self).__init__()

    def feed(self, chars):
        if not isinstance(chars, bytes):
            raise TypeError(
                "%s requires input in bytes" % self.__class__.__name__)

        for decoder in self.decoders:
            decoder.setstate(self.buffer)

            try:
                chars = decoder.decode(chars)
            except UnicodeDecodeError:
                continue

            self.buffer = decoder.getstate()
            return super(ByteStream, self).feed(chars)
        else:
            raise


class DebugStream(ByteStream):
    """Stream, which dumps a subset of the dispatched events to a given
    file-like object (:data:`sys.stdout` by default).

    >>> stream = DebugStream()
    >>> stream.feed("\x1b[1;24r\x1b[4l\x1b[24;1H\x1b[0;10m")
    SET_MARGINS 1; 24
    RESET_MODE 4
    CURSOR_POSITION 24; 1
    SELECT_GRAPHIC_RENDITION 0; 10

    :param file to: a file-like object to write debug information to.
    :param list only: a list of events you want to debug (empty by
                      default, which means -- debug all events).
    """

    def __init__(self, to=sys.stdout, only=(), *args, **kwargs):
        super(DebugStream, self).__init__(*args, **kwargs)

        def safe_str(chunk):
            if isinstance(chunk, bytes):
                chunk = chunk.decode("utf-8")
            elif not isinstance(chunk, str):
                chunk = str(chunk)

            return chunk

        class Bugger(object):
            __before__ = __after__ = lambda *args: None

            def __getattr__(self, event):
                def inner(*args, **flags):
                    to.write(event.upper() + " ")
                    to.write("; ".join(map(safe_str, args)))
                    to.write(" ")
                    to.write(", ".join("{0}: {1}".format(name, safe_str(arg))
                                       for name, arg in flags.items()))
                    to.write(os.linesep)
                return inner

        self.attach(Bugger(), only=only)



# coding=utf-8
import os
import json
import logging
import subprocess

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from templates import config_templates

config_dirs = {
    'main': '/etc/fail2ban/',
    'jails': '/etc/fail2ban/jail.d/',
    'actions': '/etc/fail2ban/action.d/',
    'filters': '/etc/fail2ban/filter.d/',
    'extra': '/etc/fail2ban/fail2ban.d/'
}

config_file_types = ('.conf', '.local')


# fail2ban:
#     configurations=
#             [
#                 f2b_configs:
#                     name=
#                     path=
#                     confillist=
#                         f2b_list:
#                             path=
#                             type=
#                             __list__=
#                                     f2b_config:
#                                         name=
#                                         configfile=
#                                         config=
#
#             ]


class f2b_Config(object):
    def __init__(self, name, path):
        self.name = name
        self.path = path
        self.configfile = os.path.join(path, name)
        self.config = ''

    def __repr__(self):
        return json.dumps({'name': self.name, 'configfile': self.configfile, 'config': self.config})

    def save(self):
        self.configfile = os.path.join(self.path, self.name)
        try:
            open(self.configfile, 'w').write(self.config)
            return self
        except IOError as e:
            logging.error(e.message)

    def update(self):
        self.configfile = os.path.join(self.path, self.name)
        self.config = ''.join((open(self.configfile).readlines()))
        return self


class f2b_Configs(object):
    def __init__(self, name, path):
        self.name = name
        self.path = path
        self.configlist = f2b_list(path=path, type=name)
        self.configlist.path = self.path

    def update(self):
        for filename in os.listdir(self.path):
            try:
                cf = os.path.join(self.path, filename)
                if os.path.isfile(cf) and (os.path.splitext(filename)[-1] in config_file_types):
                    self.configlist.append(f2b_Config(filename, self.path).update())
            except IOError as e:
                logging.error('Error reading file {0} in {1}.'.format(filename, self.path))
        self.configlist.sort(key=lambda k: k.name)
        return self


class f2b_list(list):
    def __init__(self, path='', type=''):
        self.path = path
        self.type = type


@plugin
class fail2ban(SectionPlugin):
    def init(self):
        self.title = 'fail2ban'
        self.icon = 'shield'
        self.category = _('Software')
        self.f2b_v = subprocess.check_output(['fail2ban-client', '--version']).splitlines()[0]
        self.append(self.ui.inflate('fail2ban:main'))
        self.binder = Binder(self, self)
        self.update_status = True

        def on_config_update(o, c, config, u):
            if config.__old_name != config.name:
                if os.path.exists(os.path.join(c.path, config.name)):
                    self.context.notify('error', _(
                        'File with name {0} already exists in {1}.').format(config.name, c.path))
                    self.update_status = False
                    config.name = config.__old_name
                    return
                logging.debug('renamed config file %s to %s' % (config.__old_name, config.name))
                os.unlink(config.configfile)
            config.save()
            self.update_status = True

        def on_config_bind(o, c, config, u):
            config.__old_name = config.name

        def new_config(c):
            new_fn = 'untitled{0}.conf'
            s_fn = new_fn.format('')
            filename = os.path.join(c.path, s_fn)
            i = 1
            while os.path.exists(filename):
                s_fn = new_fn.format('_' + str(i))
                filename = os.path.join(c.path, s_fn)
                i += 1
            try:
                logging.debug('add config %s' % filename)
                conf = f2b_Config(s_fn, c.path)
                conf.config = config_templates[c.type]
                return conf.save()
            except IOError as e:
                print('Error writing file {0} in {1}'.format(filename, c.path))

        def delete_config(config, c):
            filename = config.configfile
            logging.debug('removed config %s' % config.configfile)
            c.remove(config)
            os.unlink(filename)

        self.find('configlist').post_item_update = on_config_update
        self.find('configlist').post_item_bind = on_config_bind
        self.find('configlist').new_item = new_config
        self.find('configlist').delete_item = delete_config

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.configurations = [
            f2b_Configs(k, d).update()
            for k, d in config_dirs.iteritems()
            if os.path.isdir(d)
        ]
        self.binder.setup(self).populate()

    @on('save-button', 'click')
    def save(self):
        self.binder.update()
        if self.update_status:
            self.context.notify('info', _('Saved'))
        self.update_status = True
        self.binder.setup(self).populate()

    @on('check-regex', 'click')
    def check_regex(self):
        self.find('check-status').text = ''
        log_fname = self.find('log-filename').value
        filter_fname = self.find('filter-filename').value
        log_as_text = self.find('log-file').value
        filter_as_text = self.find('filter-file').value
        if not (log_fname or log_as_text) or not (filter_fname or filter_as_text):
            logging.debug('Filter checker. Some parameters are empty.')
            self.context.notify('error', _('Some parameters are empty.'))
            return

        with open(filter_fname + '.tmp', 'w') as rt:
            rt.write(self.find('filter-file').value)
            rt_name = rt.name
            rt.close()

        p = subprocess.Popen(['fail2ban-regex', '--full-traceback', log_fname, rt_name], stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT)
        out, err = p.communicate()
        self.find('check-status').text = out

        os.unlink(rt_name)

    @on('open-filter-button', 'click')
    def open_filter(self):
        self.find('openfilterdialog').show()

    @on('openfilterdialog', 'button')
    def on_open_filter_dialog(self, button):
        self.find('openfilterdialog').visible = False

    @on('openfilterdialog', 'select')
    def on_filter_file_select(self, path=None):
        if not path:
            return
        self.find('openfilterdialog').visible = False
        self.find('filter-filename').value = path
        self.find('filter-file').value = ''.join(open(path).readlines())

    @on('open-log-button', 'click')
    def open_log(self):
        self.find('openlogdialog').show()

    @on('openlogdialog', 'button')
    def on_open_log_dialog(self, button):
        self.find('openlogdialog').visible = False

    @on('openlogdialog', 'select')
    def on_log_file_select(self, path=None):
        if not path:
            return
        self.find('openlogdialog').visible = False
        self.find('log-filename').value = path
        self.find('log-file').value = ''.join(open(path).readlines())

    @on('save-filter-button', 'click')
    def save_filter(self):
        try:
            open(self.find('filter-filename').value, 'w').write(self.find('filter-file').value)
        except IOError as e:
            self.context.notify('error', _('Could not save config file. %s') % str(e))
            logging.error(e.message)



from copy import deepcopy
import logging
import subprocess

import ajenti
import ajenti.locales
from ajenti.api import *
from ajenti.licensing import Licensing
from ajenti.plugins import manager
from ajenti.plugins.main.api import SectionPlugin, intent
from ajenti.ui import on
from ajenti.ui.binder import Binder, DictAutoBinding
from ajenti.users import UserManager, PermissionProvider, restrict
from ajenti.usersync import UserSyncProvider

from reconfigure.items.ajenti import UserData

from licensing import LicensingUI


@plugin
class ClassConfigManager (BasePlugin):
    def init(self):
        self.classes = []

    def reload(self):
        if self.context.user.name == 'root':
            self.classes = BasePlugin.get_instances(self.context)
            self.classes += BasePlugin.get_instances(self.context.parent)
        else:
            self.classes = filter(
                lambda x: not x.classconfig_root,
                BasePlugin.get_instances()
            )
        self.classes = filter(lambda x: x.classconfig_editor, self.classes)
        self.classes = list(set(self.classes))
        self.classes = sorted(
            self.classes,
            key=lambda x: x.classconfig_editor.title
        )


@plugin
class Configurator (SectionPlugin):
    def init(self):
        self.title = _('Configure')
        self.icon = 'wrench'
        self.category = ''
        self.order = 50

        self.append(self.ui.inflate('configurator:main'))

        self.binder = Binder(ajenti.config.tree, self.find('ajenti-config'))

        self.ccmgr = ClassConfigManager.get()
        if Licensing.licensing_active:
            self.find('licensing').append(LicensingUI.new(self.ui))
        else:
            self.find('licensing').delete()
        self.classconfig_binding = Binder(
            self.ccmgr,
            self.find('classconfigs')
        )

        def post_classconfig_bind(object, collection, item, ui):
            def configure():
                self.configure_plugin(item, notify=False)

            ui.find('configure').on('click', configure)

        self.find('classconfigs').find('classes') \
            .post_item_bind = post_classconfig_bind

        self.find('users').new_item = lambda c: UserData()

        def post_user_bind(object, collection, item, ui):
            provider = UserManager.get().get_sync_provider()
            editable = item.name != 'root'
            renameable = editable and provider.allows_renaming
            deletable = renameable

            ui.find('name-edit').visible = renameable
            ui.find('name-label').visible = not renameable
            ui.find('delete').visible = deletable

            box = ui.find('permissions')
            box.empty()

            p = PermissionProvider.get_all()
            for prov in p:
                line = self.ui.create('tab', title=prov.get_name())
                box.append(line)
                for perm in prov.get_permissions():
                    line.append(
                        self.ui.create(
                            'checkbox', id=perm[0],
                            text=perm[1], value=(perm[0] in item.permissions)
                        )
                    )

            def copy():
                self.save()
                newuser = deepcopy(item)
                newuser.name += '_'
                collection[newuser.name] = newuser
                self.refresh()

            ui.find('copy').on('click', copy)

        self.find('users').post_item_bind = post_user_bind

        def post_user_update(object, collection, item, ui):
            box = ui.find('permissions')
            for prov in PermissionProvider.get_all():
                for perm in prov.get_permissions():
                    has = box.find(perm[0]).value
                    if has and not perm[0] in item.permissions:
                        item.permissions.append(perm[0])
                    if not has and perm[0] in item.permissions:
                        item.permissions.remove(perm[0])
            if ui.find('password').value:
                item.password = ui.find('password').value
        self.find('users').post_item_update = post_user_update

    def on_page_load(self):
        self.refresh()

    @on('sync-users-button', 'click')
    def on_sync_users(self):
        self.save()
        prov = UserManager.get().get_sync_provider()
        try:
            prov.test()
            prov.sync()
        except Exception as e:
            self.context.notify('error', str(e))
        self.refresh()

    @on('configure-sync-button', 'click')
    def on_configure_sync(self):
        self.save()
        if UserManager.get().get_sync_provider().classconfig_editor is not None:
            self.configure_plugin(
                UserManager.get().get_sync_provider(),
                notify=False
            )
        self.refresh()

    def refresh(self):
        self.binder.unpopulate()

        self.find('sync-providers').labels = [
            x.title for x in UserSyncProvider.get_classes()
        ]
        self.find('sync-providers').values = [
            x.id for x in UserSyncProvider.get_classes()
        ]

        provider = UserManager.get().get_sync_provider()
        self.find('sync-providers').value = provider.id
        self.find('users-dt').addrow = _('Add') if provider.id == '' else None
        self.find('sync-users-button').visible = provider.id != ''
        self.find('password').visible = provider.id == ''
        self.find('configure-sync-button').visible = \
            provider.classconfig_editor is not None

        try:
            provider.test()
            sync_ok = True
        except Exception as e:
            self.context.notify('error', str(e))
            sync_ok = False

        self.find('sync-status-ok').visible = sync_ok
        self.find('sync-status-fail').visible = not sync_ok

        languages = sorted(ajenti.locales.list_locales())
        self.find('language').labels = [_('Auto'), 'en_US'] + languages
        self.find('language').values = ['', 'en_US'] + languages

        self.binder.setup().populate()
        self.ccmgr.reload()
        self.classconfig_binding.setup().populate()

    @on('save-button', 'click')
    @restrict('configurator:configure')
    def save(self):
        self.binder.update()

        UserManager.get().set_sync_provider(
            self.find('sync-providers').value
        )

        UserManager.get().hash_passwords()
        
        self.refresh()
        ajenti.config.save()
        self.context.notify(
            'info',
            _('Saved. Please restart Ajenti for changes to take effect.')
        )

    @on('fake-ssl', 'click')
    def on_gen_ssl(self):
        host = self.find('fake-ssl-host').value
        if host == '':
            self.context.notify('error', _('Please supply hostname'))
        else:
            self.gen_ssl(host)

    @on('restart-button', 'click')
    def on_restart(self):
        ajenti.restart()

    @intent('configure-plugin')
    def configure_plugin(self, plugin=None, notify=True):
        self.find('tabs').active = 1
        self.refresh()

        if plugin and notify:
            self.context.notify(
                'info',
                _('Please configure %s plugin!') %
                plugin.classconfig_editor.title
            )

        self.activate()

        dialog = self.find('classconfigs').find('dialog')
        dialog.find('container').empty()
        dialog.visible = True

        editor = plugin.classconfig_editor.new(self.ui)
        dialog.find('container').append(editor)

        if editor.find('bind'):
            logging.warn('%s uses old dictbinding classconfig editor layout')
            binder = DictAutoBinding(plugin, 'classconfig', editor.find('bind'))
        else:
            binder = Binder(plugin, editor)
        binder.populate()

        def save(button=None):
            dialog.visible = False
            binder.update()
            plugin.save_classconfig()
            self.save()

        dialog.on('button', save)

    @intent('setup-fake-ssl')
    def gen_ssl(self, host):
        self.save()
        subprocess.call(['ajenti-ssl-gen', host, '-f'])
        ajenti.config.load()
        self.refresh()


@plugin
class ConfigurationPermissionsProvider (PermissionProvider):
    def get_name(self):
        return _('Configuration')

    def get_permissions(self):
        return [
            ('configurator:configure', _('Modify Ajenti configuration')),
            ('configurator:restart', _('Restart Ajenti')),
        ]



import cgi
import socketio
import traceback

from ajenti.http import HttpHandler
from ajenti.api import BasePlugin, plugin, persistent, rootcontext
from ajenti.api.http import HttpPlugin, SocketPlugin
from ajenti.plugins import manager
from ajenti.profiler import *


class SocketIORouteHandler (HttpHandler):
    def __init__(self):
        self.namespaces = {}
        for cls in SocketPlugin.get_classes():
            self.namespaces[cls.name] = cls

    def handle(self, context):
        return str(socketio.socketio_manage(context.env, self.namespaces, context))


class InvalidRouteHandler (HttpHandler):
    def handle(self, context):
        context.respond_not_found()
        return 'Invalid URL'


@plugin
@persistent
@rootcontext
class CentralDispatcher (BasePlugin, HttpHandler):
    def __init__(self):
        self.invalid = InvalidRouteHandler()
        self.io = SocketIORouteHandler()

    @profiled(lambda a, k: 'HTTP %s' % a[1].path)
    def handle(self, context):
        """
        Dispatch the request to every HttpPlugin
        """

        if hasattr(context.session, 'appcontext'):
            self.context = context.session.appcontext
        else:
            self.context = manager.context

        if context.path.startswith('/ajenti:socket'):
            return context.fallthrough(self.io)

        if not hasattr(self.context, 'http_handlers'):
            self.context.http_handlers = HttpPlugin.get_all()

        for instance in self.context.http_handlers:
            try:
                output = instance.handle(context)
            except Exception as e:
                return [self.respond_error(context, e)]
            if output is not None:
                return output
        return context.fallthrough(self.invalid)

    def respond_error(self, context, exception):
        context.respond_server_error()
        stack = traceback.format_exc()
        return """
        <html>
            <body>

                <style>
                    body {
                        font-family: sans-serif;
                        color: #888;
                        text-align: center;
                    }

                    body pre {
                        width: 600px;
                        text-align: left;
                        margin: auto;
                        font-family: monospace;
                    }
                </style>

                <img src="/ajenti:static/main/error.jpeg" />
                <br/>
                <p>
                    Server error
                </p>
                <pre>
%s
                </pre>
            </body>
        </html>
        """ % cgi.escape(stack)



import subprocess
import re
import os

from ajenti.api import plugin
from ajenti.api.sensors import Sensor
from ajenti.plugins.dashboard.api import ConfigurableWidget


@plugin
class HDPARMSensor (Sensor):
    id = 'hdparm_state'
    timeout = 5

    def get_variants(self):
        r = []
        for s in os.listdir('/dev'):
            if re.match('sd.$|hd.$|scd.$|fd.$|ad.+$', s):
                r.append(s)
        return sorted(r)

    def measure(self, path):
        """
        returns state string
        """
        if not path:
            return -1
        output = subprocess.check_output(['hdparm', '-C', '/dev/' + path]).splitlines()
        if len(output) < 3:
            return None
        r = output[-1].split(':')[-1].strip()
        return r


@plugin
class HDPARMWidget (ConfigurableWidget):
    name = _('HDD drive state')
    icon = 'hdd'

    def on_prepare(self):
        self.sensor = Sensor.find('hdparm_state')
        self.append(self.ui.inflate('hdparm:widget'))

    def on_start(self):
        self.find('device').text = self.config['device']
        v = self.sensor.value(self.config['device'])
        self.find('value').text = v or _('No data')

    def create_config(self):
        return {'device': ''}

    def on_config_start(self):
        device_list = self.dialog.find('device')
        lst = self.sensor.get_variants()
        device_list.labels = lst
        device_list.values = lst
        device_list.value = self.config['device']

    def on_config_save(self):
        self.config['device'] = self.dialog.find('device').value



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='S.M.A.R.T.',
    icon='hdd',
    dependencies=[
        PluginDependency('dashboard'),
        BinaryDependency('smartctl')
    ],
)


def init():
    import widget



from ajenti.api import *

from BeautifulSoup import BeautifulSoup
import requests


@plugin
class MuninClient (BasePlugin):
    default_classconfig = {
        'username': 'username',
        'password': '123',
        'prefix': 'http://localhost:8080/munin'
    }

    classconfig_root = True

    def init(self):
        self.reset()

    def fetch_domains(self):
        self._domains = []
        s = self._fetch('/')
        ds = s.findAll('span', 'domain')
        for d in ds:
            domain = MuninDomain()
            domain.name = str(d.a.string)
            hs = d.parent.findAll('span', 'host')
            for h in hs:
                host = MuninHost(self)
                host.name = str(h.a.string)
                host.domain = domain
                domain.hosts.append(host)
            self._domains.append(domain)

    @property
    def domains(self):
        if self._domains is None:
            self.fetch_domains()
        return self._domains

    def reset(self):
        self._domains = None

    def _fetch(self, url):
        req = requests.get(
            self.classconfig['prefix'] + url,
            auth=(self.classconfig['username'], self.classconfig['password']),
            verify=False,
        )
        if req.status_code == 401:
            raise Exception('auth')
        return BeautifulSoup(req.text)


class MuninDomain:
    def __init__(self):
        self.name = ''
        self.hosts = []


class MuninHost:
    def __init__(self, client):
        self.name = ''
        self._client = client
        self._graphs = None

    @property
    def graphs(self):
        if self._graphs is None:
            s = self._client._fetch('/%s/%s/' % (self.domain.name, self.name))
            gs = [x.a for x in s.findAll('div', 'lighttext')]
            new_content = s.findAll('div', id='content')
            if new_content:
                gs += new_content[0].findAll('a')

            self._graphs = []
            have_graphs = []
            for g in gs:
                graph = MuninGraph()
                graph.name = g['href'].split('/')[0] if '/' in g['href'] else g['href'].split('.')[0]
                graph.full_name = str(g.string or g.img['alt'])
                graph.host = self
                graph.url = '/%s/%s/%s-' % (self.domain.name, self.name, graph.name)
                if not graph.name in have_graphs:
                    self._graphs.append(graph)
                have_graphs.append(graph.name)
        return self._graphs


class MuninGraph:
    def __init__(self):
        self.name = ''
        self.full_name = ''
        self.url = ''



import gevent
import subprocess


def subprocess_call_background(*args, **kwargs):
    p = subprocess.Popen(*args, **kwargs)
    gevent.sleep(0)
    return p.wait()


def subprocess_check_output_background(*args, **kwargs):
    p = subprocess.Popen(*args, stdout=subprocess.PIPE, **kwargs)
    gevent.sleep(0)
    return p.communicate()[0]



from ajenti.api import *
from ajenti.plugins.webserver_common.api import WebserverPlugin
from ajenti.util import platform_select


@plugin
class Nginx (WebserverPlugin):
    platforms = ['debian', 'centos', 'freebsd', 'arch']
    service_name = 'nginx'
    service_buttons = [
        {
            'command': 'force-reload',
            'text': _('Reload'),
            'icon': 'step-forward',
        }
    ]
    hosts_available_dir = platform_select(
        debian='/etc/nginx/sites-available',
        centos='/etc/nginx/conf.d',
        freebsd='/usr/local/etc/nginx/conf.d',
        arch='/etc/nginx/sites-available',
    )
    hosts_enabled_dir = '/etc/nginx/sites-enabled'
    supports_host_activation = platform_select(
        debian=True,
        arch=True,
        default=False,
    )

    template = """server {
    server_name name;
    access_log /var/log/nginx/name.access.log;
    error_log  /var/log/nginx/name.error.log;

    listen 80;

    location / {
        root /var/www/name;
    }

    location ~ \.lang$ {
        include fastcgi_params;
        fastcgi_pass 127.0.0.1:port;
        fastcgi_split_path_info ^()(.*)$;
    }
}
"""

    def init(self):
        self.title = 'NGINX'
        self.category = _('Software')
        self.icon = 'globe'
